import json
from dataclasses import dataclass, fields, field, is_dataclass, asdict, _MISSING_TYPE


@dataclass
class Datamodel:
    @classmethod
    def decoder(cls, data: object):
        obj = cls()
        if isinstance(data, dict):
            for f in fields(cls):
                if f.name not in data:  # 该字段不在 data key 中
                    continue
                elif f.type.__module__ == 'typing':  # 处理来自 typing 模块的类型
                    setattr(obj, f.name, parse(f.type, data[f.name]))
                elif is_dataclass(f.type):  # 处理 dataclass 类
                    setattr(obj, f.name, f.type.decoder(data[f.name]))
                else:  # 处理基本或其他数据类型
                    if data[f.name] is None:  # None则为该字段的默认值
                        isDefMis = isinstance(f.default, _MISSING_TYPE)
                        setattr(obj, f.name, f.default_factory if isDefMis else f.default)
                    else:
                        try:  # 否则尝试转换为该字段类型
                            setattr(obj, f.name, f.type(data[f.name]))
                        except Exception as err:
                            if not (f.type in (int, float) and data.get(f.name) == ''):
                                print(f'{cls.__name__} {f.name} decoder error: {err}')
            return obj
        elif isinstance(data, str):
            try:
                return cls.decoder(json.loads(data))
            except json.decoder.JSONDecodeError:
                raise
        elif isinstance(data, (list, tuple, set)):  # 过滤掉非字典元素
            return [cls.decoder(x) for x in data if isinstance(x, dict)]
        elif data is not None:
            raise ValueError("不支持的数据类型！")

    def asdict(self) -> dict:
        return asdict(self)


def parse(T, data) -> object:
    if T.__module__ == 'typing' and not T._special:
        if args := T.__args__:
            if T._name == 'List':
                arg = args[0]  # List 的参数类型
                if arg.__module__ == 'typing' and not arg._special:
                    return parse(arg.__args__[0], data)
                return parse(arg, data)
            elif T._name == 'Dict':
                return {k: parse(T.__args__[1], v) for k, v in data.items()}
    if is_dataclass(T):
        if isinstance(data, list):
            return [T.decoder(x) for x in data]
        return T.decoder(data)
    return data

